/* qcam.h -- routines for accessing the Connectix QuickCam */

/* Version 0.1, January 2, 1996 */
/* Version 0.5, August 24, 1996 */
/* Version 0.7, August 26, 1996 */


/******************************************************************

Copyright (C) 1996 by Scott Laird

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL SCOTT LAIRD BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

/* types of QuickCams: */
#define QCAM_BW				0x01
#define QCAM_COLOR			0x10

/* commands common to all quick-cameras: */
#define QC_SEND_VIDEO_FRAME		 7
#define QC_SET_BRIGHTNESS		11
#define QC_SET_TOP			13
#define QC_SET_LEFT			15
#define QC_SET_NUM_V			17
#define QC_SET_NUM_H			19
#define QC_SEND_VERSION			23
#define QC_SET_BLACK			29
#define QC_SET_WHITE			31
#define QC_SET_SATURATION		35
#define QC_SEND_STATUS			41
#define QC_SET_SPEED			45

/* commands for grayscale camera: */
#define QC_BW_SET_CONTRAST		25
#define QC_BW_AUTO_ADJUST_OFFSET	27
#define QC_BW_GET_OFFSET		33

/* commands for color camera: */
#define QC_COL_LOAD_RAM			27
#define QC_COL_SET_HUE			33
#define QC_COL_SET_CONTRAST		37

/* One from column A... */
#define QC_NOTSET 0
#define QC_UNIDIR 1
#define QC_BIDIR  2
#define QC_SERIAL 3

/* ... and one from column B */
#define QC_ANY          0x00
#define QC_FORCE_UNIDIR 0x10
#define QC_FORCE_BIDIR  0x20
#define QC_FORCE_SERIAL 0x30
/* in the port_mode member */

#define QC_MODE_MASK    0x07
#define QC_FORCE_MASK   0x70

struct qcam {
  int cam_version;
  int width, height;
  int bpp;
  int mode;
  int contrast, brightness, whitebal, blacklevel, saturation, hue, speed;
  int port;
  int port_mode;
  int transfer_scale;
  int top, left;
  int fd; /* lock file descriptor
           * It was, unfortunately, necessary to add this member to the
           * struct qcam to conveniently implement POSIX fcntl-style locking.
           * We need a seperate lock file for each struct qcam, for instance,
           * if the same process (using qcam-lib) is accessing multiple
           * QuickCams on (of course) multiple ports.
           * - Dave Plonka (plonka@carroll1.cc.edu)
           */
};

typedef unsigned char scanbuf;

/* General QuickCam handling routines */

int qc_getbrightness(struct qcam *q);
int qc_setbrightness(struct qcam *q, int val);
int qc_sethue(struct qcam *q, int val);
int qc_setsaturation(struct qcam *q, int val);
int qc_getcontrast(struct qcam *q);
int qc_setcontrast(struct qcam *q, int val);
int qc_getwhitebal(struct qcam *q);
int qc_setwhitebal(struct qcam *q, int val);
int qc_getblacklevel(struct qcam *q);
int qc_setblacklevel(struct qcam *q, int val);
void qc_getresolution(struct qcam *q, int *x, int *y);
int qc_setresolution(struct qcam *q, int x, int y);
int qc_getbitdepth(struct qcam *q);
int qc_setbitdepth(struct qcam *q, int val);
int qc_getheight(struct qcam *q);
int qc_setheight(struct qcam *q, int y);
int qc_getwidth(struct qcam *q);
int qc_setwidth(struct qcam *q, int x);
int qc_gettop(struct qcam *q);
int qc_settop(struct qcam *q, int val);
int qc_getleft(struct qcam *q);
int qc_setleft(struct qcam *q, int val);
int qc_gettransfer_scale(struct qcam *q);
int qc_settransfer_scale(struct qcam *q, int val);
int qc_calibrate(struct qcam *q);
int qc_forceunidir(struct qcam *q);

struct qcam *qc_init(void);
int qc_initfile(struct qcam *q, char *fname);
int qc_open(struct qcam *q);
int qc_close(struct qcam *q);
int qc_detect(struct qcam *q);
void qc_reset(struct qcam *q);
void qc_set(struct qcam *q);
scanbuf *qc_scan(struct qcam *q);
scanbuf *qc_convertscan(struct qcam *q, scanbuf *scan);
void qc_writepgm(struct qcam *q, FILE *f,scanbuf *scan);
void qc_writeppm(struct qcam *q, FILE *f, scanbuf *scan);
void qc_wait(int val);

/* OS/hardware specific routines */

int read_lpstatus(struct qcam *q);
int read_lpcontrol(struct qcam *q);
int read_lpdata(struct qcam *q);
void write_lpdata(struct qcam *q, int d);
void write_lpcontrol(struct qcam *q, int d);
int enable_ports(struct qcam *q);
int disable_ports(struct qcam *q);
int qc_unlock(struct qcam *q);
int qc_lock(struct qcam *q);
void qc_wait(int val);
int qc_probe(struct qcam *q);

#define INCL_DOSPROCESS
#include <os2.h>


extern _Far16 _Pascal OUTP8(USHORT, UCHAR);
extern _Far16 _Pascal OUTP16(USHORT, USHORT);
extern UCHAR _Far16 _Pascal INP8(USHORT);
extern USHORT _Far16 _Pascal INP16(USHORT);

#define  usleep(x)   DosSleep(x)
#define inb(x)       INP8(x)
#define outb(x,y)    OUTP16(x,y)
